==> pagedesigner-2.0.0-rc1.tar.gz: pagedesigner/modules/pagedesigner_debug/src/Commands/Debug.php <==
<?php

namespace Drupal\pagedesigner_debug\Commands;

use Drupal\user\Entity\User;
use Drupal\pagedesigner_debug\Service\StructureDebugger;
use Drush\Commands\DrushCommands;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * A drush command file.
 *
 * @package Drupal\pagedesigner_debug\Commands
 */
class Debug extends DrushCommands {

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger = NULL;

  /**
   * The data structure debugger.
   *
   * @var \Drupal\pagedesigner_debug\Service\StructureDebugger
   */
  protected $debugger = NULL;

  /**
   * The print mask.
   *
   * @var string
   */
  protected $mask = "| %-10.10s | %-30.30s | %-60.60s |\n";

  /**
   * Creates a new Debug instance.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger channel factory.
   * @param \Drupal\pagedesigner_debug\Service\StructureDebugger $debugger
   *   The debugger.
   */
  public function __construct(LoggerChannelFactoryInterface $logger, StructureDebugger $debugger) {
    parent::__construct();
    $this->logger = $logger->get('pagedesigner_debug');
    $this->debugger = $debugger;
  }

  /**
   * Drush command that displays the given text.
   *
   * @command pagedesigner_debug:check
   * @aliases pd_debug:check
   * @option strategy
   *   The strategy to employ: 'preserve' or 'cleanup'. Defaults to preserve.
   * @usage pagedesigner_debug:correct --strategy=preserve
   */
  public function check($options = ['strategy' => 'preserve']) {
    $this->debugger->setStrategy($options['strategy']);
    if (!$this->io()->confirm('The debugger is experimental! Do you want to continue?')) {
      return FALSE;
    }
    $this->doCheck();
    $this->io()->comment('Debugging stopped.');
  }

  /**
   * Drush command that displays the given text.
   *
   * @command pagedesigner_debug:correct
   * @aliases pd_debug:correct
   * @option strategy
   *   The strategy to employ: 'preserve' or 'cleanup'. Defaults to preserve.
   * @usage pagedesigner_debug:correct --strategy=preserve
   */
  public function correct($options = ['strategy' => 'preserve']) {
    $this->debugger->setStrategy($options['strategy']);
    if (!$this->io()->confirm('The debugger is experimental! Do you want to continue?')) {
      return FALSE;
    }
    do {
      $hasCorrections = $this->doCorrect();
      if (empty($hasCorrections) && $hasCorrections !== FALSE) {
        $this->io()->title('Clearing up special elements');
        $this->debugger->clearSpecialElements();
        $this->io()->title('Removing orphans');
        if (($orphanCount = $this->debugger->removeOrphans(TRUE)) !== FALSE) {
          $this->io()->success('Removed ' . $orphanCount . ' orphans.');
          $hasCorrections = $this->doCorrect();
        }
        else {
          $this->io()->warning('Removing orphans failed.');
        }
      }
    } while ($hasCorrections);
    $this->io()->comment('Debugging stopped.');
  }

  /**
   * Executes check runs.
   */
  protected function doCheck() {
    $this->io()->title('Collecting corrections');
    $fields = \Drupal::entityTypeManager()->getStorage('field_storage_config')->loadByProperties([
      'entity_type' => 'node',
      'type' => 'pagedesigner_item',
    ]);
    $bundles = [];
    /** @var \Drupal\field\FieldConfigStorage $field_storage */
    foreach ($fields as $field_storage) {
      if (!empty($field_storage)) {
        $bundles = array_merge($bundles, array_keys($field_storage->getBundles()));
      }
    }

    $nids = \Drupal::entityQuery('node')
      ->condition('type', $bundles, 'IN')
      ->addMetaData('account', User::load(1))
      ->execute();
    $this->debugger->checkStructure($nids);
    $errors = $this->debugger->getErrors();
    $corrections = $this->debugger->getCorrections();
    if (!empty($errors)) {
      $this->printErrors($errors);
      if (!$this->io()->confirm('Do you want to continue despite errors?', FALSE)) {
        return FALSE;
      }
    }
    if (!empty($corrections)) {
      $this->printCorrections($corrections);
    }
    else {
      $this->io()->success('No more corrections found.');
    }
    return $corrections;
  }

  /**
   * Helper method to apply corrections.
   *
   * @return array
   *   The corrections found.
   */
  protected function doCorrect() {
    while (($corrections = $this->doCheck())) {
      if ($this->io()->confirm('Do you want to apply the listed corrections?')) {
        $this->io()->title('Applying corrections');
        $this->debugger->applyCorrections();
      }
      else {
        return FALSE;
      }
    }
    return $corrections;
  }

  /**
   * Print the found corrections.
   */
  protected function printErrors($errors) {
    $this->io()->warning('There are errors which need to be corrected by hand.');
    $tableData = [];
    foreach ($errors as $element => $data) {
      foreach ($data as $type => $value) {
        $text = (is_array($value)) ? implode(',', $value) : $value;
        $tableData[] = [$element, $type, $text];
      }
    }
    $this->io->table(['Element', 'Error', 'Values'], $tableData);
  }

  /**
   * Print the found corrections.
   */
  protected function printCorrections(array $corrections) {
    $tableData = [];
    foreach ($corrections as $element => $data) {
      foreach ($data as $type => $value) {
        $text = (is_array($value)) ? implode(',', $value) : $value;
        $tableData[] = [$element, $type, $text];
      }
    }
    $this->io->table(['Element', 'Correction', 'Values'], $tableData);
  }

}

==> pagedesigner-2.0.2.tar.gz: pagedesigner/modules/pagedesigner_debug/src/Commands/Debug.php <==
<?php

namespace Drupal\pagedesigner_debug\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\pagedesigner_debug\Service\StructureDebugger;
use Drush\Commands\DrushCommands;

/**
 * A drush command file.
 *
 * @package Drupal\pagedesigner_debug\Commands
 */
class Debug extends DrushCommands {

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger = NULL;

  /**
   * The data structure debugger.
   *
   * @var \Drupal\pagedesigner_debug\Service\StructureDebugger
   */
  protected $debugger = NULL;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager = NULL;

  /**
   * The print mask.
   *
   * @var string
   */
  protected $mask = "| %-10.10s | %-30.30s | %-60.60s |\n";

  /**
   * Creates a new Debug instance.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger channel factory.
   * @param \Drupal\pagedesigner_debug\Service\StructureDebugger $debugger
   *   The debugger.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(LoggerChannelFactoryInterface $logger, StructureDebugger $debugger, EntityTypeManagerInterface $entity_type_manager) {
    $this->logger = $logger->get('pagedesigner_debug');
    $this->debugger = $debugger;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Drush command that displays the given text.
   *
   * @command pagedesigner_debug:check
   * @aliases pd_debug:check
   * @option strategy
   *   The strategy to employ: 'preserve' or 'cleanup'. Defaults to preserve.
   * @usage pagedesigner_debug:correct --strategy=preserve
   */
  public function check($options = ['strategy' => 'preserve']) {
    $this->debugger->setStrategy($options['strategy']);
    if (!$this->getConsent()) {
      return FALSE;
    }
    $this->doCheck();
    $this->io()->comment('Debugging stopped.');
  }

  /**
   * Drush command that displays the given text.
   *
   * @command pagedesigner_debug:correct
   * @aliases pd_debug:correct
   * @option strategy
   *   The strategy to employ: 'preserve' or 'cleanup'. Defaults to preserve.
   * @usage pagedesigner_debug:correct --strategy=preserve
   */
  public function correct($options = ['strategy' => 'preserve']) {
    $this->debugger->setStrategy($options['strategy']);
    if (!$this->getConsent()) {
      return FALSE;
    }
    \Drupal::state()->set('pagedesigner_debug_running', TRUE);
    do {
      $hasCorrections = $this->doCorrect();
      if (empty($hasCorrections) && $hasCorrections !== FALSE) {
        $this->io()->title('Clearing up special elements');
        $this->debugger->clearSpecialElements();
        $this->io()->title('Removing orphans');
        if (($orphanCount = $this->debugger->removeOrphans(TRUE)) !== FALSE) {
          $this->io()->success('Removed ' . $orphanCount . ' orphans.');
          $hasCorrections = $this->doCorrect();
        }
        else {
          $this->io()->warning('Removing orphans failed.');
        }
      }
    } while ($hasCorrections);
    \Drupal::state()->delete('pagedesigner_debug_running');
    $this->io()->comment('Debugging stopped.');
  }

  /**
   * Executes check runs.
   */
  protected function doCheck() {
    $this->io()->title('Collecting corrections');
    $fields = $this->entityTypeManager->getStorage('field_storage_config')->loadByProperties([
      'entity_type' => 'node',
      'type' => 'pagedesigner_item',
    ]);
    $bundles = [];
    /** @var \Drupal\field\FieldStorageConfigInterface $field */
    foreach ($fields as $field) {
      if (!empty($field)) {
        $bundles = array_merge($bundles, array_keys($field->getBundles()));
      }
    }

    $nids = \Drupal::entityQuery('node')
      ->condition('type', $bundles, 'IN')
      ->accessCheck(FALSE)
      ->execute();
    $this->debugger->checkStructure($nids);
    $errors = $this->debugger->getErrors();
    $corrections = $this->debugger->getCorrections();
    if (!empty($errors)) {
      $this->printErrors($errors);
      if (!$this->io()->confirm('Do you want to continue despite errors?', FALSE)) {
        return FALSE;
      }
    }
    if (!empty($corrections)) {
      $this->printCorrections($corrections);
    }
    else {
      $this->io()->success('No more corrections found.');
    }
    $orphans = $this->debugger->findOrphans();
    if (!empty($orphans)) {
      $this->io()->note('Found ' . count($orphans) . ' orphans.');
    }
    else {
      $this->io()->success('No orphans found.');
    }
    return $corrections;
  }

  /**
   * Helper method to apply corrections.
   *
   * @return array
   *   The corrections found.
   */
  protected function doCorrect() {
    while (($corrections = $this->doCheck())) {
      if ($this->io()->confirm('Do you want to apply the listed corrections?')) {
        $this->io()->text('Applying corrections (this might take a while)');
        $this->debugger->applyCorrections();
      }
      else {
        return FALSE;
      }
    }
    return $corrections;
  }

  /**
   * Print the found corrections.
   */
  protected function printErrors($errors) {
    $this->io()->warning('There are errors which need to be corrected by hand.');
    $tableData = [];
    foreach ($errors as $element => $data) {
      foreach ($data as $type => $value) {
        $text = (is_array($value)) ? implode(',', $value) : $value;
        $tableData[] = [$element, $type, $text];
      }
    }
    $this->io->table(['Element', 'Error', 'Values'], $tableData);
  }

  /**
   * Print the found corrections.
   */
  protected function printCorrections(array $corrections) {
    $tableData = [];
    foreach ($corrections as $element => $data) {
      foreach ($data as $type => $value) {
        $text = (is_array($value)) ? implode(',', $value) : $value;
        $tableData[] = [$element, $type, $text];
      }
    }
    $this->io->table(['Element', 'Correction', 'Values'], $tableData);
  }

  /**
   * Get consent from user to continue.
   *
   * @return bool
   *   True, if the user gives consent.
   */
  protected function getConsent() {
    $this->io()->warning('The debugger is experimental!');
    return $this->io()->confirm('Do you want to continue?');
  }

}
